<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/action/action.class.php" );
    include_once( PLOG_CLASS_PATH."class/action/actionform.class.php" );
    include_once( PLOG_CLASS_PATH."class/action/actioninfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/object/reflection.class.php" );
    include_once( PLOG_CLASS_PATH."class/object/exception.class.php" );

	/**
     * This is how MVC works, using the pattern of the 'Front Controller'. With this pattern, we have
     * a single controller class that receives all the requests from the client, identifies the
     * action to be taken and the relays the execution of the action to the most suitable Action class.
     * The 'Action' class then will use the application business logic (the Model) to carry out the
     * operations necessary.
     *
     * (according to http://java.sun.com/blueprints/guidelines/designing_enterprise_applications_2e/web-tier/web-tier5.html)
     *
     * 1. The controller receives a POST from the client.
	 * 2. The controller creates an Action corresponding to the requested operation (as described in the previous section).
 	 * 3. The controller calls the Action's perform method.
	 * perform calls a model business method.
	 * 4. The controller calls the screen flow manager to select the next view to display.
	 * 5. The screen flow manager determines the next view and returns its name to the controller.
	 * 6. The controller forwards the request to the templating service, which assembles and delivers the selected view to the client.
     *
     * In our particular case, we have two kinds of Actions: Action and ActionForm. The first is the
     * one that should be normally used, while the second should be used when receiving data from
     * forms, since it provides an additional method, validate() that will allow the developer
     * to validate the data that came from the form. If the result of validate() is 'false',
     * the controller will <b>not</b> call the perform() method and stop execution. However, the
     * validate method should also generate a valid view containing probably the error message.
     */

	class Controller extends Object {

		var $_actionMap;
        var $_actionParam;
        var $_loadActionClasses;

    	/**
         * $ActionsMap is an associative array of the form:
         *
         * ( $actionName, $actionClassName )
         *
         * Where for every different possible value of the 'action' parameter in the request,
         * there is an object inheriting form the Action class that will take care of
         * that requested action.
         *
         * @param actionMap is the associative array with the mappings
         * @param actionParam is the name of the parameter in the request that will be used
         * @param loadActionClasses By default set to 'true', enables dynamic loading of the
         * action classes from disk. Set it to false if the action classes that are going
         * to be needed by the controller have already been loaded and there is no need to
         * do it again.
         * to identify the action to be taken
         */
		function Controller( $actionMap, $actionParam = "action", $loadActionClasses = true )
        {
			$this->Object();

			$this->_actionMap = $actionMap;
            $this->_actionParam = $actionParam;
            $this->_loadActionClasses = $loadActionClasses;
        }

        /**
         * Processess the HTTP request sent by the client
         *
         * @param httpRequest HTTP request sent by the client
         */
        function process( $httpRequest )
        {
            // get the name of the action
            $request = new Request( $httpRequest );
            $actionName = $request->getValue($this->_actionParam );


            // If there is no action, we use the default one (Default)
            // If the action does not exist, we use the default again (Default)
            if(($actionName == "") || (!array_key_exists($actionName, $this->_actionMap)))
            	$actionName = "Default";

            // get the object for that action
            $actionClass = $this->_actionMap[$actionName];

			// and finally execute it
            if( $this->_loadActionClasses ) {
	    	$fileName = PLOG_CLASS_PATH."class/action/".strtolower($actionClass).".class.php";
            	include( $fileName );
            }
            $actionInfo   = new ActionInfo( $this->_actionParam, $actionName );
            $actionObject = new $actionClass( $actionInfo, $httpRequest );

            // if the action is a child of ActionForm, first we let the object execute
			// the validate method to check the values of the form variables. If validate
            // returns 'true', then we call the 'perform' method
            //if( $actionObject->isSubclass( new ActionForm("", "")) ) {
            $reflect = new Reflection();
            if( $reflect->methodExists( $actionObject, "validate")) {
            	if( $actionObject->validate())
                	$actionObject->perform();
            }
            else {
            	$actionObject->perform();
            }

            // once the operation has been carried out, fetch and send the view
            // to the client
            $view = $actionObject->getView();

            // this is not good...
            if( $view == null || $view == "" ) {
            	$e = new Exception( "The view is empty after calling the perform method." );
                throw( $e );
            }
            else
            	$view->render();
        }
    }
?>
