<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
	include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/userinfo.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/bloginfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategory.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );

	define( "PUBLISHED", 'published' );
    define( "DRAFT", 'draft' );
    define( "DELETED", 'deleted' );

    define( "POST_EXTENDED_TEXT_MODIFIER", "[@more@]" );

    /**
     * This class represents an article from the database, and provides methods to access all its objects.
     */
	class Article extends Object {

        /**
         * Private members. Use the getXXX classes to access them
         * @private
         */
		var $_id;
		var $_topic;
		var $_text;
		var $_categoryId;
        var $_category;
		var $_user;
		var $_date;
		var $_timestamp;
		var $_comments;
		var $_userInfo;
		var $_blog;
		var $_blogInfo;
		var $_status;
        var $_totalComments;
        var $_numReads;
        var $_numTrackbacks;
        var $_properties;
		var $_timeOffset;

        /**
         * Constructor.
         *
         * @param topic Topic of the article
         * @param text Text of the article
         * @param category Identifier of the category under which the article will be included
         * @param user Identifier of the user who posted the article
         * @param blog Identifier of the blog to which the article belongs
         * @param status Can be either PUBLISHED, DRAFT or DELETED
         * @param numReads How many times the article has been read.
         * @param properties An associative array containing some settings for the post,
         * such as if comments are enabled/disabled, if it's a private or public post,
         * or its password.
         * @param id If present, the identifier of the article in the database
         */
		function Article( $topic, $text, $category, $user, $blog, $status, $numReads, $properties = Array(), $id = -1 )
		{
			$this->_topic    = $topic;
			$this->_text     = $text;
			$this->_categoryId = $category;
			$this->_user     = $user;
			$this->_blog     = $blog;
			$this->_id       = $id;
			$this->_status   = $status;
			$this->_comments = "";
            $this->_totalComments = 0;
            $this->_numTrackbacks = 0;
            $this->_numReads = $numReads;
			// by default it'll have current time
			//$this->_timestamp = new Timestamp();
			//$this->_date      = $this->_timestamp->getTimestamp();
			$this->_timestamp = 0;
			$this->_date = 0;
            $this->_properties = $properties;
			$this->_timeOffset = 0;

            //print_r($this);
		}

        /**
         * Returns the topic of the article
         * @return The topic
         */
		function getTopic()
		{
			return $this->_topic;
		}

        /**
         * Returns the text of the article
         * @return The text
         */
		function getText( $replace = true )
		{
        	if( $replace )
				return str_replace( POST_EXTENDED_TEXT_MODIFIER, "<br/>", $this->_text );
            else
            	return $this->_text;
		}

        function getIntroText()
        {
        	$postParts = explode( POST_EXTENDED_TEXT_MODIFIER, $this->_text );
            return $postParts[0];
        }

        function getExtendedText()
        {
        	$postParts = explode( POST_EXTENDED_TEXT_MODIFIER, $this->_text );
            return $postParts[1];
        }

        function hasExtendedText()
        {
            if($this->getExtendedText() == "<br />" || $this->getExtendedText() == "<br/>" ||
               $this->getExtendedText() == "<p/>" || $this->getExtendedText() == "<p />")
            	return false;
        	else
            	return( strlen($this->getExtendedText()) > 0 );
        }

	   /**
         * Returns the category of the article
         * @return The category identifier
         */
		function getCategoryId()
		{
			return $this->_categoryId;
		}

        /**
         * Returns the ArticleCategory object.
         *
         * @return The ArticleCategory object.
         */
        function getCategory()
        {
        	return $this->_category;
        }

        /**
         * Returns the identifier assigned to this article.
         *
         * @return An integer value representing the identifier.
         */
		function getId()
		{
			return $this->_id;
		}

        /**
         * The identifier of the user who posted this comment
         *
         * @return An integer value representing the user idenfitier.
         * @see getUserInfo
         */
		function getUser()
		{
			return $this->_user;
		}

        /**
         * Returns the date, 14-digit format straight from the database.
         *
         * @return The 14-digit representation of the date.
         */
		function getDate()
		{
			return $this->_date;
		}

        /**
         * Returns an array of UserComment objects, with all the comments that have been received for
         * this article.
         *
         * @return An array of UserComment objects.
         * @see UserComment
         */
		function getComments()
		{
			return $this->_comments;
		}

        /**
         * Returns the UserInfo object containing information about the owner of this post.
         *
         * @return A UserInfo object.
         * @see UserInfo
         */
		function getUserInfo()
		{
			return $this->_userInfo;
		}


        /**
         * Returns the blog identifier to which this article belongs.
         *
         * @return An integer value representing the blog identifier.
         * @see getBlogInfo
         */
		function getBlog()
		{
			return $this->_blog;
		}

        /**
         * Returns the status of the article:
         * <ul><li>published</li><li>draft</li><li>deleted</li></ul>
         *
         * @return A string value representing the status of the post.
         */
		function getStatus()
		{
			return $this->_status;
		}

        /**
         * Returns the BlogInfo object with information regarding the blog to which this
         * article belongs.
         *
         * @return The BlogInfo object.
         * @see BlogInfo
         */
		function getBlogInfo()
		{
			return $this->_blogInfo;
		}

        /**
         * Returns the number of times this article has been visited.
         *
         * @return An integer value representing the number of times this article has been read.
         */
        function getNumReads()
        {
        	return $this->_numReads;
        }

        /**
         * Returns the properties array
         *
         * @return An associative array
         */
        function getProperties()
        {
        	return $this->_properties;
        }

        function getCommentsEnabled()
        {
        	$commentsEnabled = $this->_properties["comments_enabled"];

            if( $commentsEnabled == "" )
            	$commentsEnabled = false;

            return $commentsEnabled;
        }

        /**
         * @private
         */
		function setUserInfo( $newUserInfo )
		{
			$this->_userInfo = $newUserInfo;
		}

        /**
         * @private
         */
		function setTopic( $newTopic )
		{
			$this->_topic = $newTopic;
		}

        /**
         * @private
         */
		function setText( $newText )
		{
			$this->_text = $newText;
		}

        /**
         * Changes the category id
         * @private
         */
		function setCategoryId( $newCategory )
		{
			$this->_categoryId = $newCategory;
		}

        /**
         * @private
         */
        function setCategory( $category )
        {
        	$this->_category = $category;
        }

        /**
         * @private
         */
		function setId( $newId )
		{
			$this->_id = $newId;
		}

        /**
         * @private
         */
		function setUser( $newUser )
		{
			$this->_user = $newUser;
		}

        /**
         * @private
         */
		function setComments( $newComments )
		{
			$this->_comments = $newComments;
		}

        /**
         * @private
         */
		function setBlog( $blog )
		{
			$this->_blog;
		}

        /**
         * @private
         */
		function setStatus( $status )
		{
			$this->_status = $status;
		}

        /**
         * Returns true wether this post has the 'DRAFT' status.
         *
         * @return True if the post is a draft or false otherwise.
         */
        function isDraft()
        {
        	return( $this->_status == DRAFT );
        }

        /**
         * Returns true wether this post has been deleted or not (status == DELETED)
         *
         * @return True if the post as deleted or false otherwise.
         */
        function isDeleted()
        {
        	return( $this->_status == DELETED );
        }

        /**
         * Returns true wether this post has been published or not (status == PUBLISHED)
         *
         * True if the post has been published or false otherwise.
         */
        function isPublished()
        {
        	return( $this->_status == PUBLISHED );
        }

        /**
         * Returns the number of comments that this post has (only the number!!)
         *
         * @return An integer containing the number of comments that this post has.
         */
		function getTotalComments()
		{
			//if( $this->_comments != null )
			//	return $this->_comments->getTotalComments();
            return $this->_totalComments;
		}

        /**
         * Returns the number of trackback pings that this post has received.
         *
         * @return An integer representing the number of trackback pings.
         */
        function getNumTrackbacks()
        {
        	return $this->_numTrackbacks;
        }

        /**
         * @private
         */
        function setTotalComments( $totalComments )
        {
        	$this->_totalComments = $totalComments;
        }

        /**
         * @private
         */
        function setNumTrackbacks( $numTrackbacks )
        {
        	$this->_numTrackbacks = $numTrackbacks;
        }

        /**
         * @private
         */
		function setDate( $newDate )
		{
			$this->_date = $newDate;

			$this->_timestamp = new Timestamp( $newDate );
		}

        /**
         * Commodity function that returns the year when this post was posted.
         *
         * @return The 4-digit representation of the year.
         */
		function getYear()
		{
			return $this->_timestamp->getYear();
		}

        /**
         * Commodity function that returns the month when this post was posted.
         *
         * @return The 2-digit representation of the month.
         */
		function getMonth()
		{
			return $this->_timestamp->getMonth();
		}

        /**
         * Commodity function that returns the day when this post was posted.
         *
         * @return The 2-digit representation of the day of the month.
         */
		function getDay()
		{
			return $this->_timestamp->getDay();
		}

        /**
         * Commodity function that returns the hour of the day when this post was posted.
         *
         * @return The 2-digit-24-hour representation of the hour.
         */
		function getHour()
		{
			return $this->_timestamp->getHour();
		}

        /**
         * Commodity function that returns the minute when this post was posted.
         *
         * @return The 2-digit representation of the minute this post was posted.
         */
		function getMinutes()
		{
			return $this->_timestamp->getMinutes();
		}

        /**
         * Returns the Timestamp object representing the date when this post was posed.
         *
         * @see Timestamp
         * @return A Timestamp object representing the date.
         */
		function getDateObject()
		{
			return $this->_timestamp;
		}

        /**
         * @private
         */
		function setBlogInfo( $blogInfo )
		{
			$this->_blogInfo = $blogInfo;
		}

        /**
         * @private
         */
        function setNumReads( $numReads )
        {
        	$this->_numReads = $numReads;
        }

        /**
         * sets a new Timestamp object for this post, so that we can
         * change the date of the post if necessary.
         */
        function setDateObject( $newTimestamp )
        {
        	$this->_timestamp = $newTimestamp;
            $this->_date      = $this->_timestamp->getTimestamp();
        }

        function setProperties( $properties )
        {
        	$this->_properties = $properties;
        }

        function setCommentsEnabled( $commentsEnabled )
        {
        	$this->_properties["comments_enabled"] = $commentsEnabled;

            return true;
        }
		
		function setTimeOffset( $timeOffset )
		{
			$this->_timeOffset = $timeOffset;
		}
		
		function getTimeOffset()
		{
			return $this->_timeOffset;
		}
	}
?>
