<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryalbum.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresources.class.php" );

    /**
     * Database access for GalleryAlbum objects.
	 *
	 * Please keep in mind that in the context of this library, every album has to have an user who
	 * owns it. When this library is used within pLog, users don't own albums but blogs do so we will
	 * use blog identifier instead of user identifiers. The reason for this change on how things are
	 * called was that this library can also be used outside pLog, and outside pLog we will most likely
	 * not have blogs but users.
	 *
	 * @see GalleryAlbum
     */
    class GalleryAlbums extends Model
    {

    	var $_childAlbums;
        var $resources;

    	function GalleryAlbums()
        {
        	$this->Model();
            $this->_childAlbums = Array();
        }

        /**
         * Returns an array with all the albums that belong to the given
         * owner
         *
         * @param ownerId The identifier of the owner whose albums we'd like to fetch
         * @param onlyShownAlbums Returns only those albums that have the show_album
         * field set to true, or all of them otherwise
         * @return An array containing GalleryAlbum objects, representing the
         * albums we fetched from the db.
         */
        function getUserAlbums( $ownerId, $onlyShownAlbums = false )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."gallery_albums WHERE owner_id = $ownerId";
            if( $onlyShownAlbums )
            	$query .=" AND show_album = 1";

            return $this->_getAlbumsFromQuery( $query );
        }

		/**
		 * Returns a specific album from the database.
		 *
		 * @param id A valid album identifier.
		 * @param ownerId (optional) The album should have the given id _and_ it should
		 * belong to the given user.
		 * @param fetchResources (optional) Whether at the same time, we should also fetch the
		 * resources that are associated to this album. Set it to 'false' if you only need
		 * to access the album and do not need to use methods such as GalleryAlbum::getResources()
		 * to fetch the resources that have been categorized under this album. It of course
		 * saves resources and database queries.
		 * @param onlyShownAlbums Forces to retrieve only the albums that have been set to appear
		 * in the main page. If set to 'true', we will generate an error if the album exists, 
		 * has the specified id and belongs to the specified user _but_ it is not supposed to be 
		 * shown in the main page.
		 * @return Returns a valid GalleryAlbum object or 'false' if the album could not be found.
		 */
        function getAlbum( $id, $ownerId = -1, $fetchResources = true, $onlyShownAlbums = false )
        {
        	// make sure that we don't have this album yet...
            if( isset($this->cache[$id])) {
            	return $this->cache[$id];
            }

            // otherwise, continue the normal process...
        	$query = "SELECT * FROM ".$this->getPrefix()."gallery_albums WHERE id = $id";
            if( $ownerId != -1 )
            	$query .= " AND owner_id = $ownerId";
            if( $onlyShownAlbums )
            	$query .= " AND show_album = 1";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            if( $result->RecordCount() == 0 )
            	return false;

            $row = $result->FetchRow();
            $album = $this->_fetchAlbumInformation( $row, $fetchResources, $onlyShownAlbums );

            // ...and keep a copy of the object we just fetched
            // just in case, one  never knows :)
            $this->cache[$id] = $album;

            return $album;
        }

        /**
         * Returns an array with all the child albums of the given album, but only
         * the ones at the first level (it is not recursive!)
         *
         * @param albumId The album identfitier whose children we'd like to get.
		 * @param ownerId To whom should this album belong.
         * @param onlyShownAlbums Returns only those albums that have the show_album
         * field set to true, or all of them otherwise
         * @return An array of GalleryAlbum objects
         */
        function getChildAlbums( $albumId, $ownerId, $onlyShownAlbums = false )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."gallery_albums
                      WHERE parent_id = $albumId AND
					  owner_id = $ownerId";
            if( $onlyShownAlbums )
            	$query .= " AND show_album = 1";

            return $this->_getAlbumsFromQuery( $query );
        }

        /**
         * @private
         */
        function _getAlbumsFromQuery( $query )
        {
            $result = $this->_db->Execute( $query );

            // return empty array if the query didn't succeed
            if( !$result )
            	return Array();

            $albums = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $albums, $this->_fetchAlbumInformation( $row ));
            }

            return $albums;
        }

        /**
         * Adds an album to the database
         *
         * @param ownerId The owner to which the album belongs
         * @param name The name of the album
         * @param description Description of the album.
		 * @param parentId The id of the parent album.
		 * @return Returns true if successful or false otherwise.
         */
        function addAlbum( $ownerId, $name, $description, $parentId )
        {
        	$query = "INSERT INTO ".$this->getPrefix()."gallery_albums (
                      owner_id, description, name, flags, parent_id, properties, show_album )
                      VALUES (
                      $ownerId, '".Db::qstr($description)."', '".Db::qstr($name)."', 0, $parentId, '".
                      serialize($properties)."', 1);";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * @private
         */
        function getNumChildren( $albumId )
        {
        	$query = "SELECT COUNT(*) AS num_children FROM ".$this->getPrefix()."gallery_albums
                      WHERE parent_id = $albumId";
            $result = $this->_db->Execute( $query );
            if( !$result )
            	return 0;

            $row = $result->FetchRow();

            return $row["num_children"];
        }

        /**
         * updates an album in the db
         *
         * @param album A GalleryAlbum object that already exists in the db.
		 * @param Returns true if successful or false otherwise.
         */
        function updateAlbum( $album )
        {
        	$query = "UPDATE ".$this->getPrefix()."gallery_albums SET
                      description = '".Db::qstr($album->getDescription())."',
                      name = '".Db::qstr($album->getName())."',
                      parent_id = ".$album->getParentId().",
                      properties = '".serialize($album->getProperties())."',
                      show_album = ".$album->getShowAlbum()."
                      WHERE id = ".$album->getId().";";

             return $this->_db->Execute( $query );
        }

        /**
         * removes an album from the db
         *
         * @param albumId The album identifier
         * @param ownerId The user identifier to whom the album belongs (optional)
		 * @return True if successful or false otherwise.
         */
        function deleteAlbum( $albumId, $userId = -1 )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."gallery_albums
                      WHERE id = $albumId";
            if( $userId != -1 )
            	$query .= " AND owner_id = $userId";

            return $this->_db->Execute( $query );
        }

		/**
		 * @private
		 */
        function _getNumResources( $albumId )
        {
        	$query = "SELECT COUNT(*) AS total FROM ".$this->getPrefix()."gallery_resources
                      WHERE album_id = $albumId";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return 0;

            $row = $result->FetchRow();

            if( $row["total"] == "" )
            	return 0;
            else
            	return $row["total"];
        }

        /**
         * @private
         */
        function _fetchAlbumInformation( $row, $fetchResources = true, $onlyShownAlbums = false )
        {
        	$album = new GalleryAlbum( $row["owner_id"],
                                       $row["name"],
                                       $row["description"],
                                       $row["flags"],
                                       $row["parent_id"],
                                       $row["date"],
                                       unserialize($row["properties"]),
                                       $row["show_album"],
                                       $row["id"] );

            // fetch the number of children and the GalleryAlbum child
            // objects that "hang" from this one
            //$album->setNumChildren( $this->getNumChildren($row["id"]));
			$id = $row["id"];
			$ownerId = $row["owner_id"];
			if( !isset($this->_childAlbums[$id])) {
				$this->_childAlbums[$id] = $this->getChildAlbums( $id, $ownerId );
			}
            $album->setChildren( $this->_childAlbums[$id] );

            $album->setNumResources( $this->_getNumResources( $id ));

            // fetch the resources that belong to this album
            /*if( $fetchResources ) {
				_debug("fetching resources for album id = $id<br/>");
				if( $this->resources == "" )
					$this->resources = new GalleryResources();				
                $albumResources = $this->resources->getUserResources( $row["owner_id"], $id );
                $album->setResources( $albumResources );
            }*/

            return $album;
        }
    }
?>
