<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
	include_once( PLOG_CLASS_PATH."class/locale/locale.class.php" );

	/**
     * Class that supports methods such as getAvailableLocales, addLocale
     * removeLocale and so on, basically to deal with locale files.
     *
     * It also provides a singleton-like getLocale method to load and cache locale
     * files from disk, so that we don't have to fetch the same file as many times
     * as we ask for it but else, we keep a cached copy of it for later use.
     * @see Locales::getLocale
     * @see Locale
     */
	class Locales extends Object {

    	function Locales()
        {
        	$this->Object();
        }

        /**
         * @static
         * Static method that offers some kind of locale factory. Since the Locale object
         * better not use a Singleton (otherwise we couldn't use more than one locale file
         * at a time) this function has been included here to provide a system similar to
         * a singleton: we keep an static array inside the function, that contains all the
         * locale files that have been loaded so far. Whenever somebody requests a locale
         * to be fetched from disk, we will first check that we have not loaded it before. If
         * we have, then we only have to return the same object we were keeping.
         * If the locale wasn't there, we will then load it from disk and store/cache the
         * resulting object for future use.
         * It is recommended to use this method over creating new Locale objects every time
         * we need one.
         *
         * @param localeCode The code (eg. en_UK, es_ES) of the locale we want to get.
         * @return Returns a Locale object corresponding to the requested locale.
         * @see Locale
         */
        function getLocale( $localeCode = null )
        {
        	// array to keep track of the locales that we have already loaded, so that
            // we don't have to fetch them from disk
        	static $loadedLocales;

            // if there is no locale parameter, we use the default one
        	if( $localeCode == null ) {
            	$config = Config::getConfig();
                $localeCode = $config->getValue( "default_locale" );
            }

            // check if we have already loaded that locale or else, load it from
            // disk and keep it for later, just in case anybody asks again
            if( isset($loadedLocales[$localeCode] )) {
            	$locale = $loadedLocales[$localeCode];
            }
            else {
            	$locale = new Locale( $localeCode );
                $loadedLocales[$localeCode] = $locale;

            }

            return $locale;
        }

        /**
         * Returns an array with the codes of the locale files available.
         *
         * @return An array containing all the locale codes available in the system.
         */
        function getAvailableLocales()
        {
        	$config = Config::getConfig();

            return $config->getValue( "locales" );
        }

        /**
         * Returns true if the given locale code is a valid one (i.e. if it is amongst
         * the available ones
         *
         * @param localeCode The code of the locale we'd like to check
         * @return Returns 'true' if the locale file is already available in the system
         * or false otherwise.
         */
        function isValidLocale( $localeCode )
        {
        	$availableLocales = $this->getAvailableLocales();

            return in_array( $localeCode, $availableLocales );
        }

        /**
         * Returns an array with all the locales available in the system.
         *
         * This is quite memory and disk-intensive since we are loading a lot of lines
         * from possibly a lot of files!!!
         *
         * @return Returns an array of Locale objects, which represent *all* the locales
         * that have been installed in this system.
         */
        function getLocales()
        {
        	$localeCodes = $this->getAvailableLocales();

            $locales = Array();
            foreach( $localeCodes as $code ) {
            	array_push( $locales, new Locale( $code ));
            }

            return $locales;
        }

        /**
         * Given a locale code, returns the path to the file that contains it.
         *
         * @param localeCode The code representing the locale
         * @return A string containing the path (may be absolute or relative, it depends)
         * to the file containing the given locale. It does *not* check if the file exists
         * or not, simply returns its path.
         */
        function getLocaleFilename( $localeCode )
        {
        	return (Locale::getLocaleFolder()."/locale_".$localeCode.".php");
        }

        /**
         * Removes a locale from the system. First the file containing it is deleted
         * and then we also remove its entry from the configuration.
         *
         * @param localeCode The code of the locale that we would like to delete.
         * @return Returns true if the locale was removed succesfully or false otherwise.
         */
        function removeLocale( $localeCode )
        {
        	$config =& Config::getConfig();

            // if we don't have permissions on the folder where the locale files
            // are stored, we don't even have to bother about it...
            if(!File::isWritable(Locale::getLocaleFolder()))
            	return false;

            $fileName = $this->getLocaleFilename( $localeCode );

            if( !unlink( $fileName ))
            	return false;

            $availableLocales = $config->getValue( "locales" );

            $newLocaleList = Array();
            foreach( $availableLocales as $locale ) {
            	if( $locale != $localeCode )
                	array_push( $newLocaleList, $locale );
            }

            $config->saveValue( "locales", $newLocaleList );

            return true;
        }

        /**
         * Adds a new locale to the list of available/supported locale. It checks to make
         * sure that the file exists and that has proper access permissions.
         *
         * @param localeCode the code of the locale we'd like to add.
         * @return Returns true if the file was successfully added or false otherwise.
         */
        function addLocale( $localeCode )
        {
        	$config = Config::getConfig();

            // if the locale is already there, there is no need to add it,
            // since the user is *maybe* just updating the file.
            if( $this->isValidLocale( $localeCode ))
            	return true;

            $fileName = $this->getLocaleFilename( $localeCode );

            // make sure that the file exists and can be read and
            // give up if not so
            if( !File::isReadable( $fileName ))
            	return false;

            // otherwise, we can happily add it to the list of supported locales
            $availableLocales = $this->getAvailableLocales();
            array_push( $availableLocales, $localeCode );
            $config->saveValue( "locales", $availableLocales );

            return true;
        }
    }
?>