<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/template.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogsettings.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/misc/glob.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/file.class.php" );

    define( "BLOG_BASE_TEMPLATE_FOLDER", "blog_" );

    /**
     * Generic template rendering service that takes care of providing the Template objects
     * whenever requested.
     *
     * The advantage of using this TemplateService class is that we can delegate on it things
     * like finding the folder where the template is, choosing the right template depending
     * on the client type (normal browser, wap-enabled device, etc)
     *
     * In order to find the most suitable template, it takes several things into account:
     *
     * <ul>
     * <li>Settings stored for the current blog</li>
     * <li>User agent of the client</li> (<b>NOTE: </b>will not be implemented yet)
     * <li>The default template specified in the server-wide configuration file</li>
     * </ul>
     */
    class TemplateService extends Object {

    	/**
         * Constructor
         */
    	function TemplateService()
        {
			$this->Object();

        }

        /**
         * Generates a Template object for the given template name. This fuction does <b>not</b>
         * require the full path to the file!!
         *
         * @param templateName The name of the template we would like to get
         * @param layout A predefined layout style
         * @param blogInfo If this parameter is not null, then it will be used to locate
         * a blog-specific template. Otherwise, we will only look for the template
         * in the "global" folders.         
         * @return A Template object representing the template file we asked for.
         */
        function Template( $templateName, $layout, $blogInfo = null )
        {
            // build the file name
            if( $blogInfo == null ) {
            	$templateFileName = $layout."/".$templateName.".template";
            }
            else {
            	//
            	// might be the case that the template is not local but global, so
                // by default, global templates will always have preference over
                // local templates. If the template is global, then
                //
                $globalTemplateFolder = $this->getBaseTemplateFolder()."/".$layout;
                $localTemplateFolder  = $this->getBaseTemplateFolder()."/".BLOG_BASE_TEMPLATE_FOLDER.$blogInfo->getId()."/".$layout;

                //print("local = $localTemplateFolder - global = $globalTemplateFolder<br/>");
                $templateFileName = $layout."/".$templateName.".template";
                if( !File::isDir( $globalTemplateFolder )) {
                	//$templateFileName = $layout."/".$templateName.".template";
                    $templateFolder = $this->getBaseTemplateFolder()."/".BLOG_BASE_TEMPLATE_FOLDER.$blogInfo->getId();
                }
            }

            // create the template and make sure if we we have to force Smarty
            // to look for it somewhere else other than the default folder
            $t = new Template( $templateFileName );
            if( $templateFolder != "" )
            	$t->setTemplateDir( $templateFolder );

            $t->assign( "templatename", $templateName );

            return $t;
        }

        /**
         * Returns an array that contains the name of all the 'global' templates
         * available.
         *
         * @return Returns an array of strings
         */
        function getGlobalTemplates()
        {
        	$config =& Config::getConfig();

            return $config->getValue( "templates" );
        }

        /**
         * Returns a list with all the custom templates available *only*
         * to this blog.
         *
         * @return An array of string containing the identifiers of the template
         * sets available for the given blog
         */
        function getBlogTemplates( $blogId )
        {
        	$blogs = new Blogs();
            $blogSettings = $blogs->getBlogSettings( $blogId );

            $templates = $blogSettings->getValue( "blog_templates" );

	    if( $templates == "" || !$templates )
	    	$templates = Array();

            return $templates;
        }

        /**
         * Returns an array with all the global templates and the custom ones
         * from the given blog.
         *
         * @return Array
         * @see Config::getGlobalTemplates
         * @see Config::getBlogTemplates
         */
        function getAllTemplates( $blogId )
        {
        	$globalTemplates = $this->getGlobalTemplates();
            $blogTemplates   = $this->getBlogTemplates( $blogId );

            $allTemplates = Array();
            foreach( $globalTemplates as $template )
            	array_push( $allTemplates, $template );
            foreach( $blogTemplates as $template )
            	array_push( $allTemplates, $template );

            return $allTemplates;
        }

        /**
         * Returns the path to where the given template should be. If blogId == 0,
         * then the template will be considered global or else it will be considered
         * as a custom template for the given blog.
         *
         * @return Returns a string with the path to the folder.
         */
        function getTemplateFolder( $templateName, $blogId = 0 )
        {
            $config =& Config::getConfig();
            $baseFolder = $config->getValue( "template_folder" );
            if( $baseFolder[strlen($baseFolder)-1] != "/" )
            	$baseFolder = $baseFolder."/";

        	if( $blogId == 0 ) {
            	$templateFolder = $baseFolder.$templateName."/";
            }
            else {
            	$templateFolder = $baseFolder.BLOG_BASE_TEMPLATE_FOLDER."$blogId/".$templateName."/";
            }

            return $templateFolder;
        }

        function getBaseTemplateFolder()
        {
        	$config =& Config::getConfig();

            $basePath = $config->getValue( "template_folder" );

            return $basePath;
        }

        function getBlogBaseTemplateFolder( $blogId )
        {
            $config =& Config::getConfig();
            $baseFolder = $config->getValue( "template_folder" );
            if( $baseFolder[strlen($baseFolder)-1] != "/" )
            	$baseFolder = $baseFolder."/";

            $templateFolder = $baseFolder.BLOG_BASE_TEMPLATE_FOLDER."$blogId/".$templateName."/";

            return $templateFolder;
        }

        /**
         * Recursively removes a folder from disk.
         *
         * @return True if successful or false otherwise.
         */
        function _removeFolder( $folderName )
        {
            // we need to get a list of the files that are inside that folder
			if( !File::isReadable( $folderName ))
				return true;
			
            $files = Glob::Glob( $folderName );

            foreach( $files as $file ) {
				if( File::isReadable( $file ))
					File::delete( $file );
            }

            $result = File::delete( $folderName );

            return $result;
        }

        function _removeTemplateFromArray( $array, $key )
        {
            $resultArray = Array();
            foreach( $array as $elem ) {
            	if( $elem != $key )
                	array_push( $resultArray, $elem );
            }

            return $resultArray;
        }

        /**
         * Removes a global template
         */
        function removeGlobalTemplate( $templateName )
        {
        	$config =& Config::getConfig();

            // remove the folder from disk
            $templateFolder = $this->getTemplateFolder( $templateName );
            $result = $this->_removeFolder( $templateFolder );
			if( !$result )
				return false;

            // remove the entry for that template from the configuration
            $availableTemplates = $config->getValue( "templates" );
            $newTemplateList = $this->_removeTemplateFromArray( $availableTemplates, $templateName );

            // update the list of templates
            $config->saveValue( "templates", $newTemplateList );

            return $true;
        }

        /**
         * Removes a blog specific template.
         */
        function removeBlogTemplate( $templateName, $blogId )
        {
        	$config =& Config::getConfig();

            // get the settings of this blog
            $blogs = new Blogs();
            $blogSettings = $blogs->getBlogSettings( $blogId );

            // looks like the blog doesn't exist, so let's not bother...
            if( $blogSettings == false || $blogSettings == "" )
            	return false;

            // remove the folder where the template is
            $templateFolder = $this->getTemplateFolder( $templateName, $blogId );
            $result = $this->_removeFolder( $templateFolder );
            if( !$result )
            	return false;

            // remove the entry for that template from the configuration
            $blogTemplates = $blogSettings->getValue( "blog_templates" );
            $newTemplateList = $this->_removeTemplateFromArray( $blogTemplates, $templateName );

            // finally, save the settings
            $blogSettings->setValue( "blog_templates", $newTemplateList );
            $blogs->updateBlogSettings( $blogId, $blogSettings );

            return true;
        }

        /**
         * Removes a template from the templates folder. If blogId == 0, then we will
         * look for it as a global template, or else we will look for the template
         * in the folder of the blog.
         *
         * @return Returns true if removed successfully or false otherwise.
         */
        function removeTemplate( $templateName, $blogId = 0 )
        {
        	if( $blogId == 0 ) {
            	$result = $this->removeGlobalTemplate( $templateName );
            }
            else {
            	$result = $this->removeBlogTemplate( $templateName, $blogId );
            }

            return $result;
        }

        /**
         * Adds a new template. If the template already exists, it is not
         * added again.
         *
         * @param templateName The name of the template we'd like to add
         * @param blogId The identifier of the blog to which we'd like to add
         * the template. If blogId == 0, then the template will be added as global.
         *
         * @return Returns true if ok or false otherwise.
         */
        function addTemplate( $templateName, $blogId = 0 )
        {
        	if( $blogId == 0 ) {
            	// the template is global
                $result = $this->addGlobalTemplate( $templateName );
            }
            else {
            	// the template is local
                $result = $this->addLocalTemplate( $templateName, $blogId );
            }

            return $result;
        }

        /**
         * Adds a global template to the site.
         *
         * @param templateName Name of the template we'd like to add
         * @return Returns true if successful or false otherwise.
         */
        function addGlobalTemplate( $templateName )
        {
        	$config =& Config::getConfig();

            // check if the value is there
            $templates = $config->getValue( "templates" );
            if( in_array( $templateName, $templates ))
            	return true;

            // if not there, we can add it
            array_push( $templates, $templateName );

            // and store the values
            $config->saveValue( "templates", $templates );

            return true;
        }

        /**
         * Adds a local template to the given blog.
         *
         * @param templateName Name that the template will use
         * @param blogId the identifier of the blog to which we're going to install
         * the new template.
         * @return True if successful or false otherwise.
         */
        function addLocalTemplate( $templateName, $blogId )
        {
        	$blogs = new Blogs();
            $blogSettings = $blogs->getBlogSettings( $blogId );
            if( $blogSettings == "" || $blogSettings == false )
            	return false;

            // get the array with the templates
            $blogTemplates = $blogSettings->getValue( "blog_templates" );

            // add a new one unless it's already there
            if( $blogTemplates == "" || $blogTemplates == null )
            	$blogTemplates = Array();

            if( in_array( $templateName, $blogTemplates ))
            	return true;

            array_push( $blogTemplates, $templateName );
            $blogSettings->setValue( "blog_templates", $blogTemplates );
            $blogs->updateBlogSettings( $blogId, $blogSettings );

            return true;
        }

        /**
         * Returns true if the template is a valid global template set, or false otherwise.
         *
         * @param templateName The name of the template.
         */
        function isTemplate( $templateName )
        {
        	$config =& Config::getConfig();
        	$templates = $config->getValue( "templates" );

            return in_array( $templateName, $templates );
        }

        /**
         * Returns true if the template is a valid blog template set, or false otherwise.
         *
         * @param templateName The name of the template.
         * @param blogId The identifier of the blog we'd like to check
         */
        function isBlogTemplate( $templateName, $blogId )
        {
        	$blogs = new Blogs();
            $blogSettings = $blogs->getBlogSettings( $blogId );

            // get the array with the template names stored in the settings
            $blogTemplates = $blogSettings->getValue( "blog_templates" );
            if( empty($blogTemplates) || $blogTemplates == false )
            	return false;

            return in_array( $templateName, $blogTemplates );
        }

        /**
         * Creates the blog-specific folder where templates are stored.
         * If the folder is not there, it will be created. If it is already there,
         * then nothing will happen.
         *
         * @param blogId The identifier of the blog.
         */
        function createBlogTemplateFolder( $blogId )
        {
            // make sure that the blog-specific folder exists
            $templateFolder = $this->getBaseTemplateFolder();
            $blogTemplateFolder = "$templateFolder/blog_".$blogId;
            if( !File::isDir( $blogTemplateFolder )) {
            	File::createDir( $blogTemplateFolder );
            }

            return $blogTemplateFolder;
        }
        
        /**
         * returns true if the given template file exists in the given template set, or false
         * if not.
         *
         * @param fileName The name of the file, without the .template extension
         * @param templateSet name of the template set.
         * @return Returns true if file exists and is readable, or false otherwise.
         */
        function templateExists( $fileName, $templateSet ) 
        {
            $filePath = $this->getBaseTemplateFolder()."/".$templateSet."/".$fileName.".template";
            
            // first check if it exists at all
            if( !File::exists( $filePath, $templateSet ))
                return false;
                
            // and if it does, check if it can be read
            if( !File::isReadable( $filePath, $templateSet ))
                return false;
                
            return true;          
        }
    }
?>